#define g1238in847_cxx

#include "TinCSn2ng.h"
#include <TH2.h>
#include <TH1F.h>
#include <TStyle.h>
#include <cstdlib>
#include <iostream>
#include <string>
#include <fstream>

class g1238in847 : public TinCSn2ng {
public:
  g1238in847(TTree * /*tree*/=0) {};
  virtual Int_t   Version() const { return 2; }
  virtual void    Begin(TTree *tree);
  virtual void    SlaveBegin(TTree *tree);
  virtual void    Init(TTree *tree);
  virtual Bool_t  Notify();
  virtual Bool_t  Process(Long64_t entry);
  virtual void    SetOption(const char *option) { fOption = option; }
  virtual void    SetObject(TObject *obj) { fObject = obj; }
  virtual void    SetInputList(TList *input) { fInput = input; }
  virtual TList  *GetOutputList() const { return fOutput; }
  virtual void    SlaveTerminate();
  virtual void    Terminate();

  UShort_t p847[16][2], p1238[16][2];
  void GetPeakRange(const char *fr);

  TH1F *harr[16][16];
  TH1F *hch[16];
  TH1F *h1238;
  TH1F *h847;

  ClassDef(g1238in847, 0);
};

void g1238in847::Begin(TTree * /*tree*/)
{
  TString option = GetOption();

  std::cout << "Initializing histograms ..." << std::endl;
  char hname[64], htitle[128];
  for(int i=0;i<16;i++) {
    sprintf(hname, "hch_%02d", i);
    sprintf(htitle, "Channel -- %02d", i);
    hch[i] = new TH1F(hname, htitle, 4096, 0, 4096);

    for(int j=0; j<16; j++) {
      if(j!=i) {
	sprintf(hname, "h_c%02d_c%02d", i, j);
	sprintf(htitle, "Channel %02d vs %02d", i, j);
	harr[i][j] = new TH1F(hname, htitle, 4096, 0, 4096);
      }
    }
  }

  h1238 = new TH1F("h1238", "Sum from all channels: 1238 keV", 4096, 0, 4096);
  h847  = new TH1F("h847", "Sum from all channels: 847 keV", 4096, 0, 4096);

  GetPeakRange("win-in-adcs.txt");
}

void g1238in847::SlaveBegin(TTree * /*tree*/) {}

Bool_t g1238in847::Process(Long64_t entry)
{
  std::cout << "Processing entry " << entry << std::endl;

  b_TDC0->GetEntry(entry);
  b_TDC1->GetEntry(entry);
  b_ADC0->GetEntry(entry);
  b_ADC1->GetEntry(entry);

  for(int i=0; i<16; i++) {
    if(ADC0_ADC0[i]>p847[i][0] && ADC0_ADC0[i]<p847[i][1]) {
      h847->Fill(ADC0_ADC0[i]); // Fill one entry to h847 sum

      for(int j=0; j<16; j++) {
	if(j!=i && ADC0_ADC0[j]>p1238[j][0] && ADC0_ADC0[j]<p1238[j][1]) {
	  harr[i][j]->Fill(ADC0_ADC0[j]);
	  hch[j]->Fill(ADC0_ADC0[j]);
	  h1238->Fill(ADC0_ADC0[j]);
	}
      }
    }
  }

  return kTRUE;
}

void g1238in847::Terminate()
{
  TFile *ftmp = new TFile("tmp.root", "RECREATE");

  ftmp->cd();
  h1238->Write();
  h847->Write();
  for(int i=0; i<16; i++) {
    hch[i]->Write();
    for(int j=0; j<16; j++)
      if(j != i) harr[i][j]->Write();
  }
  ftmp->Close();
  delete ftmp;
}

void g1238in847::SlaveTerminate() {}

void g1238in847::GetPeakRange(const char *fr)
{
  using namespace std;

  int i;
  for(i=0;i<16;i++) {
    p847[i][0] =0; p847[i][1] =0;
    p1238[i][0]=0; p1238[i][1]=0;
  }

  std::ifstream ifa(fr);
  std::cout << "Reading cuts from " << fr << std::endl;
  while(!ifa.eof()) {
    std::string line;
    getline(ifa, line);
    if(line.length()<30) continue;
    i = atoi(line.substr(6,2).c_str());
    p847[i][0]  = atoi(line.substr(10,4).c_str());
    p847[i][1]  = atoi(line.substr(15,4).c_str());
    p1238[i][0] = atoi(line.substr(21,4).c_str());
    p1238[i][1] = atoi(line.substr(26,4).c_str());
    printf("CH%02d: %d, %d; %d, %d\n", i,
	   p847[i][0], p847[i][1], p1238[i][0], p1238[i][1]);
  }

  ifa.close();
}

